<?php
/*
Plugin Name: wp Time Machine
Plugin URI: http://wordpress.org/extend/plugins/wp-time-machine/
Description: Creates an archive of all your WordPress Data & Files and then emails them along restoration instructions.
Version: 1.2.0
Author: Paul G Petty
Author URI: http://paulgriffinpetty.com/
*/

require 'includes/DropboxUploader.php';
require 'includes/S3.php';
//require 'includes/ftp.php';
                    
if (!class_exists("wpTimeMachine")) {

	class wpTimeMachine {
		
		// Constructor
		function wpTimeMachine() { 
		}
		
		function init() {
		
		  // Constants

            $wpTimeMachineVersion = "1.2.0";

            $wpcontent_dir    = ( defined('WP_CONTENT_DIR') ) ? WP_CONTENT_DIR : ABSPATH . 'wp-content';
            $wpcontent_url    = ( defined('WP_CONTENT_URL') ) ? WP_CONTENT_URL : get_option('siteurl') . '/wp-content';
            $wpplugin_dir     = ( defined('WP_PLUGIN_DIR')  ) ? WP_PLUGIN_DIR  : $wpcontent_dir . '/plugins';
            
            $wp_version       = get_bloginfo( 'version' );
            $wp_installer_url = "http://wordpress.org/wordpress-".$wp_version.".tar.gz";
            
            $wpcontent_targz  = $wpcontent_dir . "/wpTimeMachine-content-files.tar.gz";
            $wpdata_sqlgz     = $wpcontent_dir . "/wpTimeMachine-data-files.sql.gz";
            $htaccess_targz   = $wpcontent_dir . "/wpTimeMachine-htaccess.tar.gz";   
            $instructions     = $wpcontent_dir . "/wpTimeMachine-Instructions.txt";

            $files            = array( $wpcontent_targz, $wpdata_sqlgz, $htaccess_targz, $instructions );
            
            $cron_settings    = "0 0 * * * php " . $wpplugin_dir . "/wp-time-machine/wp-time-machine.php";
            
		  // Get / Set Plugin Settings
		
            $wpTimeMachineOptionsStorage = $this->getAdminOptions();

            if (isset($_POST['offsite'])) {
                $wpTimeMachineOptionsStorage['offsite'] = $_POST['offsite'];
            } 
            
            if (isset($_GET['show_info'])) {
                $wpTimeMachineOptionsStorage['show_info'] = $_GET['show_info'];
            } 

            if (isset($_POST['remote_user'])) {
                $wpTimeMachineOptionsStorage['remote_user'] = $_POST['remote_user'];
            } 

            if (isset($_POST['remote_path'])) {
                $wpTimeMachineOptionsStorage['remote_path'] = $_POST['remote_path'];
            } 

            if (isset($_POST['remote_host'])) {
                $wpTimeMachineOptionsStorage['remote_host'] = $_POST['remote_host'];
            } 
              
            if (isset($_GET['remote_pass_storage'])) {
                $wpTimeMachineOptionsStorage['remote_pass_storage'] = $_GET['remote_pass_storage'];
                
                if ($_GET['remote_pass_storage'] == "false") {
                    $wpTimeMachineOptionsStorage['remote_pass'] = "";
                } 
            } 
            
            if (isset($_POST['remote_pass'])) {
                if ($wpTimeMachineOptionsStorage['remote_pass_storage'] == "true") {
                    $wpTimeMachineOptionsStorage['remote_pass'] = $_POST['remote_pass'];
                }                
            } 
                
            update_option($this->adminOptionsName, $wpTimeMachineOptionsStorage);
            
            $offsite = $wpTimeMachineOptionsStorage['offsite'];
            
          // Strings based on offsite 
          
            switch ($offsite) {
        
                case "dropbox":
                
                    $remote_user_label = "Email";
                    $remote_pass_label = "Password";
                    $remote_path_label = "Directory";
                    
                    $offsite_name = "Dropbox";
                    
                    break;
        
                case "aws_s3":

                    $remote_user_label = "S3 Key";
                    $remote_pass_label = "S3 Secret";
                    $remote_path_label = "Bucket";
                                        
                    $offsite_name = "Amazon S3";
                    
                    break;

                /*
                case "ftp":
                    
                    break;
                */
                    
                default:
                
                    // Nothing happened ... 

            }
  
          // begin plugin UI ...
    		?>
    		
    		<script type="text/javascript" src="http://www.google.com/jsapi"></script>
    		<script type="text/javascript">
    		if (!jQuery) google.load( "jquery","1.3.2" );
    		</script>
    		<script type="text/javascript">
    		
    		var dropbox_labels = new Array("Email","Password","Directory");
    		var s3_labels = new Array("S3 Key","S3 Secret","Bucket");
    		
    		function wpTimeMachine_toggle( key, negative, positive ) {

                if ( jQuery("#"+key).attr("value") == "false" ) {
                    var val = true;
                    jQuery("#"+key).text( negative );                    
                } else {
                    var val = false;
                    jQuery("#"+key).text( positive );
                }
                
                jQuery("#"+key).attr("value", val);
                
                if (key == "show_info") {
                    jQuery("#Info").toggle();
                }
                    
                jQuery.ajax({ 
                    data:    "page=wp-time-machine.php&"+key+"="+val,
                    url:     "options-general.php",
                    success: function () {
                        return false;
                    }
                });
    		
    		}

    		jQuery(document).ready( function(){

            	jQuery("#show_info").click( function (event) {
                    event.preventDefault();
                    wpTimeMachine_toggle( "show_info", "Remove instructional text?", "Display instructional text?" );
            	});
            	
            	jQuery("#remote_pass_storage").click( function (event) {
                    event.preventDefault();
                    wpTimeMachine_toggle( "remote_pass_storage", "Don't save my <?=$remote_pass_label?>.", "Save my <?=$remote_pass_label?>." );
            	});
            	
            	jQuery(".rd").click( function() {
            	
                    var offsite = jQuery(this).val();
                    var storage_text = jQuery("#remote_pass_storage").text();
                    
                    switch (offsite) {
                    
                        case "aws_s3":
                            jQuery("label[for='remote_user']").text( s3_labels[0]+": ");
                            jQuery("label[for='remote_pass']").text( s3_labels[1]+": ");
                            jQuery("label[for='remote_path']").text( s3_labels[2]+": ");
                            storage_text = storage_text.replace( dropbox_labels[1], s3_labels[1] );
                            jQuery("#remote_pass_storage").text(storage_text);
                            break;
                        
                        case "dropbox":
                            jQuery("label[for='remote_user']").text( dropbox_labels[0]+": ");
                            jQuery("label[for='remote_pass']").text( dropbox_labels[1]+": ");
                            jQuery("label[for='remote_path']").text( dropbox_labels[2]+": ");
                            storage_text = storage_text.replace( s3_labels[1], dropbox_labels[1] );
                            jQuery("#remote_pass_storage").text(storage_text);
                            break;
                        
                        //case "ftp":
                        //  break;
                        
                        default:
                            // ??

                    }           	   
            	
            	});
            
            });
                		
    		</script>
    		
    		<style type="text/css">
        		#wpTimeMachine p { margin:10px 0; padding:5px 0; }
        		#wpTimeMachine div#Info { width:650px; margin:0; padding:10px; border:2px solid #ccc; background-color:#ffc; -moz-border-radius:4px; -webkit-border-radius:4px; }
        		#wpTimeMachine div#instructional_link { width:650px; margin:0; padding:4px 20px; text-align:right; }
        		#wpTimeMachine code { border-left:2px solid #ccc; margin:10px 20px; padding:0 5px; display: block; }
        		#wpTimeMachine ol { margin-left: 20px; }
        		#wpTimeMachine em { font-size: 11px; }
        		#wpTimeMachine fieldset { width:650px; margin-bottom:15px; padding: 10px 0; border-bottom:1px solid #e7e7e7; line-height:32px; }
        		#wpTimeMachine fieldset a { font-size:10px; }
        		#wpTimeMachine label { width:225px; display: inline-block; text-align:right; }
        		#wpTimeMachine input { width:225px; }
        		#wpTimeMachine input.cb { width:auto;  }
        		#wpTimeMachine input.rd { width:auto; margin:0 10px 0 210px; }
        		#wpTimeMachine input.sb { width:auto; height:auto; margin:0 0 0 210px; font-weight:bold; font-size:14px !important; }
        		#wpTimeMachine .hint { margin:5px 0; padding:5px; border: 1px solid green; background-color:pink; width: auto; -moz-border-radius:4px; -webkit-border-radius:4px; }
    		</style>

            <div class="wrap" id="wpTimeMachine">

                <h2>wp Time Machine, version: <? echo $wpTimeMachineVersion; ?></h2>

            <?
            
            if ($wpTimeMachineOptionsStorage['show_info'] != "true") {
            
                ?>
                <div id="Info" style="display:none">
                <?
            
            } else {
            
                ?>
                <div id="Info">
                <?
            
            }
            
            ?>

                    <p>
    
                        This plugin is pretty simple. You can use Dropbox or Amazon's S3 to store WordPress.<br />
                        <br /> 
                        Need an account? Visit:<br />
                        <a href="https://www.dropbox.com/referrals/NTE0ODYwNjc5" target="_new">Dropbox</a><br />
                        <br />
                        Or, for an S3 account visit:<br />
                        <a href="https://s3.amazonaws.com/" target="_new">https://s3.amazonaws.com</a>) <br />
                        <br />
                        Then just enter your credentials below & click the "Generate wp Time Machine archives" button.
                        
                    </p>
    
                    <p>
    
                        That said there are three important requirements for enjoying this plugin:<br />
                        PHP 5 is required ... this will remain a requirement.<br />
                        The system() function must be enabled; hopefully this can be avoided in the near future. <br />
                        Patience, based on bandwidth and server performance, as well as the performance of the this code itself;
                        it may take a minute or two for the backup process to complete.  It also depends on how much you've
                        written & how many photos (and other files) you've uploaded. 
                        
                    </p>
    
                    <p>
    
                        If this directory: <code><? echo $wpcontent_dir; ?></code> is
                        writable; once you click the button below, these 5 files will appear there:
                        
                        <ol>
                            <li>wpTimeMachine-content-files.tar.gz</li>
                            <li>wpTimeMachine-data-files.sql.gz</li>
                            <li>wpTimeMachine-htaccess.tar.gz <em>[1]</em></li>
                            <li>wpTimeMachine-Instructions.txt <em>[2]</em></li>
                        </ol>
                        
                        <em>
                        [1] if you're not using .htaccess then no, it won't be created <br />
                        [2] if you're ever recovering from a catastrophic WordPress failure, this file will help<br />
                        </em>
                        
                        <div class="hint">
                        It's a good idea to store the following two items on your own, (in the same place as the files above).<br/>
                        Your copy of WordPress: <a href="<? echo $wp_installer_url;?>"><? echo $wp_installer_url;?></a><br/>
                        Your copy of wp-config.php <em>For security purposes this file is not backed up!</em>
                        </div>
    
                    </p>
    
                    <p>
                    
                        After the files are created; they'll be uploaded to <i>your</i> Dropbox & deleted.
    
                    </p>
    
                    <p>
                    
                        <!--
                        Additionally, if available, you can add this to your website's crontab to automate 
                        this process daily:<br /><code><? echo $cron_settings; ?></code><br />
                        -->
                        
                        <em>
                        Your User ID will get saved; as mentioned below your password will not, unless you 
                        check the box above.  You will have to save your password for the cron job requests to get 
                        processed (though if possible that will change).
                        </em>
    
                    </p>
                    
                </div>
                
                <div id="instructional_link">
                <? 
                if ($wpTimeMachineOptionsStorage['show_info'] == "true") {
                    echo '<a href="javascript:void(0)" id="show_info" value="true">Remove instructional text?</a><br />';
                } else { 
                    echo '<a href="javascript:void(0)" id="show_info" value="false">Display instructional text?</a><br />';
                }
                ?>
                </div>
                            
                <p>
          
                    <form method="post" action="<?php echo $_SERVER["REQUEST_URI"]; ?>&generate=1&time=<? echo time(); ?>">

                        <fieldset>
                        
                        <? 
                        
                        if ($wpTimeMachineOptionsStorage['offsite'] != "") {
                        
                            ?>
                            <label for="offsite">Current offsite service: <strong><?=$offsite_name?></strong></label> 
                            &nbsp;&nbsp;<a href="javascript:void(0)" onclick="jQuery('#offsite_selections').toggle()">Change your offsite service</a> <br />
                            <span id="offsite_selections" style="display:none">
                            <?
                            
                        } else {
                        
                            ?>
                            <label for="offsite">Select an offsite service:</label> <br />
                            <?
                        
                        }
                        
                            ?>
                            <input class="rd" type="radio" name="offsite" value="dropbox"> Dropbox (<a href="https://www.dropbox.com/referrals/NTE0ODYwNjc5" target="_new">https://dropbox.com</a>) <br />
                            <input class="rd" type="radio" name="offsite" value="aws_s3"> Amazon S3 (<a href="https://s3.amazonaws.com/" target="_new">https://s3.amazonaws.com</a>) <br />
                            <!-- <input class="rd" type="radio" name="offsite" value="ftp"> FTP <br /> -->
                            <? 
                        
                        if ($wpTimeMachineOptionsStorage['offsite'] != "") {
                        
                            ?>
                            </span>
                            <?

                        }
                        
                        ?>
                        
                        </fieldset>
                        
                        <fieldset>

                            <label for="remote_user"><?=$remote_user_label?>: </label> 
                            
                                <input type="text" name="remote_user" value="<? echo $wpTimeMachineOptionsStorage['remote_user']; ?>" /> <br />
    
                            <label for="remote_pass"><?=$remote_pass_label?>: </label> 
                            
                                <input type="password" id="remote_pass" name="remote_pass" value="<? echo $wpTimeMachineOptionsStorage['remote_pass']; ?>" /> 
                                
                                <? 
                                
                                if ($wpTimeMachineOptionsStorage['remote_pass_storage'] == "true") {
                                
                                    ?>
                                    <a href="javascript:void(0)" id="remote_pass_storage" value="true">
                                        Don't save my <?=$remote_pass_label?>.
                                    </a>
                                    <?
                                
                                } else { 
                                
                                    ?>
                                    <a href="javascript:void(0)" id="remote_pass_storage" value="false">
                                        Save my <?=$remote_pass_label?>.
                                    </a>
                                    <?
                                
                                }
                                
                                ?>
                                
                                <br />
    
                            <label for="remote_path"><?=$remote_path_label?>: </label> 
                            
                                <input type="text" name="remote_path" value="<? echo $wpTimeMachineOptionsStorage['remote_path']; ?>" /> <em>(Optional)</em> <br />
                            
                            <!--
                            <label for="remote_path">Remote Host: </label> 
                            
                                <input type="text" name="remote_host" value="<? echo $wpTimeMachineOptionsStorage['remote_host']; ?>" /> <em>(FTP only!)</em> <br />
                            -->

                        </fieldset>
                      
                        <div class="submit">
                            <input class="sb" type="submit" id="submit_package_request" value="Generate wp Time Machine archives" />
                        </div>
                        
                    </form>

                </p>

                <pre id="output" style="display:none">

                <?
                
                if ($_GET['generate'] == 1) {
                
                  // generate instructions text file
                  
                    // these instructions get generated by the plugin so that they can be easily updated &
                    // included with the backup delivery ...
                  
                    $intructions_txt = "
                    _____________________________________________________________________________
                    
                    
                        wpTimeMachine, version: ".$wpTimeMachineVersion."
                    
                    _____________________________________________________________________________
                    
                        These steps are recommendations, if you have a better way to 
                        handle restoration, consider sharing that in the forums, here:
                        
                        http://wordpress.org/tags/wp-time-machine?forum_id=10
                        
                        If you get stuck that's also a good place to search for help
                    
                    _____________________________________________________________________________
                    
                        Step 1)
                        
                            Get your old version of WordPress in place:
                            
                            ".$wp_installer_url."
                            
                            Uncompress that file after downloading it & then upload
                            it to the 'old' location where you had WordPress before.
                            
                            Just like when you first set up WordPress, you will need
                            to make a copy of wp-config-sample.php called wp-config.php
                            and update that file with your database information.  Unless
                            you manually backed this up -- which would've been a good 
                            idea.
                        
                        Step 2)
                        
                            If you had a .htaccess file in use, uncompress this file:
                            
                            wpTimeMachine-htaccess.tar.gz
                            
                            And then place it here:
                            
                            http://yourdomain.com/.htaccess
                            
                        Step 3)
                        
                            Rename the WordPress folder called wp-content to something
                            like wp-content-blank-slate.
                            
                            Uncompress wpTimeMachine-content-files.tar.gz & double check 
                            the directory structure, which should look like:
                            
                            wp-content/
                                        plugins/
                                        themes/
                                        uploads/
                                        
                            (There may be other files in there, too.)
                            
                            Upload wp-content into your WordPress folder from Step 1 ...
                            wp-content should be at the same directory level as 
                            wp-content-blank-slate
                            
                        Step 4)
                        
                            Uncompress wpTimeMachine-data-files.sql.gz ...
                            
                            This will create a .sql file which can be used to restore all
                            of your data.
                            
                            Use the same method you originally used to set up your database
                            to get your back up in place.  This most likely involved using
                            your hosting providers instance of phpMyAdmin. 
                        
                    
                    _____________________________________________________________________________
                    
                        That's it ... start testing ... 
                        
                        wpTimeMachine, version: ".$wpTimeMachineVersion."

                    _____________________________________________________________________________
                    

                    ";
                    
                    $instructions_handle = fopen($instructions, 'w');
                    fwrite($instructions_handle, $intructions_txt);
                    fclose($instructions_handle);
                  
                  // create tar files from wp-content & for htaccess
                
                    system("tar cvfz " . $wpcontent_targz . " " . $wpcontent_dir);
                    system("tar cvfz " . $htaccess_targz . " " . $_SERVER['DOCUMENT_ROOT'] . "/.htaccess");
                    
                  // create mysql dump, also a tar.gz
                  
                    system("mysqldump --opt -h " . DB_HOST . " -u " . DB_USER . " -p" \
                                    . DB_PASSWORD . " " . DB_NAME . " | gzip > " . $wpdata_sqlgz);

                  // start processing the files
                    
                    switch ($offsite) {
                
                        case "dropbox":
                        
                            $uploader = new DropboxUploader( $_POST['remote_user'], $_POST['remote_pass'] );
                            foreach ($files as $i => $file) {
                                $uploader->upload( $file, $remoteDir='/'.$_POST['remote_path'] );
                                unlink( $file );
                            }
                            
                            break;
                
                        case "aws_s3":
                        
                            $bucket = uniqid($_POST['remote_path']);
                        
                            $s3 = new S3( $_POST['remote_user'], $_POST['remote_pass'] );
                            $s3->putBucket( $bucket, S3::ACL_PUBLIC_READ );
                            foreach ($files as $i => $file) {
                                $s3->putObjectFile($file, $bucket, baseName($file), S3::ACL_PUBLIC_READ);
                                unlink( $file );
                            }
                            
                            break;

                        /*
                        case "ftp":

                            $conn_id = ftp_connect( $_POST['remote_host'] );
                            $login_result = ftp_login( $conn_id, $_POST['remote_user'], $_POST['remote_pass'] );
                            
                            foreach ($files as $i => $file) {
                            
                                $remote_file = str_replace( $wpcontent_dir, $_POST['remote_path'], $file );
                            
                                ftp_put($conn_id, $remote_file, $file, FTP_ASCII);
                                unlink( $file );
                            }
                            
                            break;
                        */
                            
                        default:
                        
                            // Nothing happened ... 

                    }

                }
                    
                ?>

                </pre><!-- close #output -->

            </div>
    		<?php
          // End plugin ui	
    		
    	} // End function init()
    	
        function  getAdminOptions() {
        
            $wpTimeMachineOptions = array(
                'offsite' => '',
                'show_info' => 'true',
                'remote_user' => 'true',
                'remote_pass_stored' => 'false',
                'remote_host' => '',
                'remote_pass' => '',
                'remote_path' => ''
            );
                
            $wpTimeMachineOptionsStorage = get_option($this->adminOptionsName);
            
            if (!empty($wpTimeMachineOptionsStorage)) {
                foreach ($wpTimeMachineOptionsStorage as $key => $option) {
                    $wpTimeMachineOptions[$key] = $option;
                }    
            }  
                      
            update_option($this->adminOptionsName, $wpTimeMachineOptions);
            
            return $wpTimeMachineOptions;
            
        } // End function getAdminOptions()
        
	}

} // End Class wpTimeMachine

// Instantiate Class
if (class_exists("wpTimeMachine")) {
	$wpTimeMachine = new wpTimeMachine();
}

// Insert link in menu for Admin Pages
if (!function_exists("wpTimeMachine_ap")) {
	function wpTimeMachine_ap() {
        global $wpTimeMachine;
        if (!isset($wpTimeMachine)) {
            return;
        }
        if (function_exists('add_options_page')) {
            add_options_page('wp Time Machine', 'wp Time Machine', 9, basename(__FILE__), array(&$wpTimeMachine, 'init'));
        }
	}	
}

// WP Actions
if (isset($wpTimeMachine)) {

	add_action('admin_menu', 'wpTimeMachine_ap');
    add_action('wp-time-machine/wp-time-machine.php',  array(&$wpTimeMachine, 'init'));
	
}

?>